@@ -1,8 +1,8 @@
 // src/cli/display.rs
 
-use crate::application::services::factory::create_interpolation_service;
 use crate::domain::bookmark::Bookmark;
 use crate::domain::search::SemanticSearchResult;
+use crate::util::helper::{format_file_path, format_mtime};
 use chrono::{DateTime, Utc};
 use derive_builder::Builder;
 use std::fmt;
@@ -46,6 +46,7 @@ pub const DEFAULT_FIELDS: &[DisplayField] = &[
     DisplayField::Title,
     DisplayField::Description,
     DisplayField::Tags,
+    DisplayField::LastUpdateTs, // shows BOTH timestamps
     DisplayField::Similarity,
 ];
 
@@ -56,7 +57,7 @@ pub const ALL_FIELDS: &[DisplayField] = &[
     DisplayField::Description,
     DisplayField::Tags,
     DisplayField::AccessCount,
-    DisplayField::LastUpdateTs,
+    DisplayField::LastUpdateTs, // shows BOTH timestamps
     DisplayField::Similarity,
     DisplayField::Embedding,
     DisplayField::Embeddable,
@@ -83,6 +84,9 @@ pub struct DisplayBookmark {
     #[builder(default = "0")]
     pub access_count: i32,
 
+    #[builder(default)]
+    pub created_at: Option<DateTime<Utc>>,
+
     #[builder(default = "chrono::Utc::now()")]
     pub last_update_ts: DateTime<Utc>,
 
@@ -94,20 +98,17 @@ pub struct DisplayBookmark {
 
     #[builder(default = "false")]
     pub embeddable: bool,
+
+    #[builder(default)]
+    pub file_path: Option<String>,
+
+    #[builder(default)]
+    pub file_mtime: Option<i32>,
 }
 
 impl DisplayBookmark {
     pub fn from_domain(bookmark: &Bookmark) -> Self {
-        // Try to render the URL if it contains interpolation markers
-        let interpolation_service = create_interpolation_service();
-        let url = if bookmark.url.contains("{{") || bookmark.url.contains("{%") {
-            match interpolation_service.render_bookmark_url(bookmark) {
-                Ok(rendered) => rendered,
-                Err(_) => bookmark.url.clone(), // Fallback to original URL if rendering fails
-            }
-        } else {
-            bookmark.url.clone()
-        };
+        let url = bookmark.url.clone();
 
         DisplayBookmarkBuilder::default()
             .id(bookmark.id.unwrap_or(0))
@@ -116,6 +117,7 @@ impl DisplayBookmark {
             .description(bookmark.description.to_string())
             .tags(bookmark.formatted_tags())
             .access_count(bookmark.access_count)
+            .created_at(bookmark.created_at) // Pass through the Option<DateTime<Utc>>
             .last_update_ts(bookmark.updated_at)
             .embedding(
                 bookmark
@@ -124,6 +126,8 @@ impl DisplayBookmark {
                     .map_or_else(String::new, |_| "yes".to_string()),
             )
             .embeddable(bookmark.embeddable)
+            .file_path(bookmark.file_path.clone())
+            .file_mtime(bookmark.file_mtime)
             .build()
             .unwrap()
     }
@@ -156,6 +160,7 @@ impl DisplayBookmark {
             .description(base.description)
             .tags(base.tags)
             .access_count(base.access_count)
+            .created_at(base.created_at)
             .last_update_ts(base.last_update_ts)
             .embedding(base.embedding)
             .embeddable(base.embeddable)
@@ -176,18 +181,21 @@ impl Default for DisplayBookmark {
             description: String::new(),
             tags: String::new(),
             access_count: 0,
+            created_at: None,
             last_update_ts: Utc::now(),
             similarity: None,
             embedding: String::new(),
             embeddable: false,
+            file_path: None,
+            file_mtime: None,
         }
     }
 }
 
 /// Display bookmarks with color formatting
-pub fn show_bookmarks(bookmarks: &[DisplayBookmark], fields: &[DisplayField]) {
+pub fn show_bookmarks(bookmarks: &[DisplayBookmark], fields: &[DisplayField], settings: &crate::config::Settings) {
     if bookmarks.is_empty() {
-        println!("No bookmarks to display");
+        eprintln!("No bookmarks to display");
         return;
     }
 
@@ -325,20 +333,45 @@ pub fn show_bookmarks(bookmarks: &[DisplayBookmark], fields: &[DisplayField]) {
             }
         }
 
-        // Last update timestamp (magenta)
+        // Created and Last update timestamps (magenta)
         if fields.contains(&DisplayField::LastUpdateTs) {
             if let Err(e) = stderr.set_color(ColorSpec::new().set_fg(Some(Color::Magenta))) {
                 eprintln!("Error setting color: {}", e);
             }
+            // Format creation date as "null" if not available
+            let created_str = match bm.created_at {
+                Some(created) => created.to_string(),
+                None => "null".to_string(),
+            };
+
             if let Err(e) = writeln!(
                 &mut stderr,
-                "{:first_col_width$}  {}",
-                "", bm.last_update_ts
+                "{:first_col_width$}  Created: {} | Updated: {}",
+                "", created_str, bm.last_update_ts
             ) {
                 eprintln!("Error writing to stderr: {}", e);
             }
         }
 
+        // File info (grey) - show if present and enabled
+        if settings.fzf_opts.show_file_info {
+            if let (Some(file_path), Some(file_mtime)) = (&bm.file_path, bm.file_mtime) {
+                if let Err(e) = stderr.set_color(ColorSpec::new().set_fg(Some(Color::Ansi256(8)))) {
+                    // Use Ansi256 color 8 which is bright black (grey)
+                    eprintln!("Error setting color: {}", e);
+                }
+                let formatted_path = format_file_path(file_path, 120);
+                let formatted_time = format_mtime(file_mtime);
+                if let Err(e) = writeln!(
+                    &mut stderr,
+                    "{:first_col_width$}  📁 {} ({})",
+                    "", formatted_path, formatted_time
+                ) {
+                    eprintln!("Error writing to stderr: {}", e);
+                }
+            }
+        }
+
         // Reset colors and print a blank line between bookmarks
         if let Err(e) = stderr.reset() {
             eprintln!("Error resetting color: {}", e);
@@ -351,10 +384,13 @@ pub fn show_bookmarks(bookmarks: &[DisplayBookmark], fields: &[DisplayField]) {
 mod display_tests {
     use super::*;
     use chrono::Utc;
-    use serial_test::serial;
     use std::{fs, path::Path};
 
     fn create_test_bookmarks() -> Vec<DisplayBookmark> {
+        // Create a common timestamp for testing
+        let now = Utc::now();
+        let earlier = now - chrono::Duration::days(30);
+
         vec![
             DisplayBookmark {
                 id: 1,
@@ -365,10 +401,13 @@ mod display_tests {
                         .to_string(),
                 tags: ",rust,programming,systems,".to_string(),
                 access_count: 42,
-                last_update_ts: Utc::now(),
+                created_at: Some(earlier), // Created 30 days ago
+                last_update_ts: now,       // Updated now
                 similarity: Some(0.85),
                 embedding: "yes".to_string(),
                 embeddable: true,
+                file_path: None,
+                file_mtime: None,
             },
             DisplayBookmark {
                 id: 2,
@@ -377,10 +416,13 @@ mod display_tests {
                 description: "The Rust Programming Language Book".to_string(),
                 tags: ",book,documentation,rust,learning,".to_string(),
                 access_count: 24,
-                last_update_ts: Utc::now(),
+                created_at: None,    // No creation date
+                last_update_ts: now, // Updated now
                 similarity: None,
                 embedding: "".to_string(),
                 embeddable: false,
+                file_path: None,
+                file_mtime: None,
             },
             DisplayBookmark {
                 id: 3,
@@ -389,25 +431,27 @@ mod display_tests {
                 description: "".to_string(), // Empty description
                 tags: ",crates,registry,".to_string(),
                 access_count: 12,
-                last_update_ts: Utc::now(),
+                created_at: Some(now), // Created now (same as updated)
+                last_update_ts: now,   // Updated now
                 similarity: Some(0.62),
                 embedding: "yes".to_string(),
                 embeddable: true,
+                file_path: None,
+                file_mtime: None,
             },
         ]
     }
 
     #[test]
-    #[serial]
-    fn test_show_bookmarks_visual() {
+    fn given_bookmarks_when_show_with_default_fields_then_displays_colored_output() {
         println!("\n\nTEST: Colored Bookmark Display - Default Fields\n");
         let bookmarks = create_test_bookmarks();
-        show_bookmarks(&bookmarks, DEFAULT_FIELDS);
+        let settings = crate::config::Settings::default();
+        show_bookmarks(&bookmarks, DEFAULT_FIELDS, &settings);
     }
 
     #[test]
-    #[serial]
-    fn test_show_bookmarks_visual_all_fields() {
+    fn given_bookmarks_when_show_with_all_fields_then_displays_extended_output() {
         println!("\n\nTEST: Colored Bookmark Display - All Fields\n");
         let bookmarks = create_test_bookmarks();
 
@@ -424,20 +468,20 @@ mod display_tests {
             DisplayField::Embedding,
         ];
 
-        show_bookmarks(&bookmarks, extended_fields);
+        let settings = crate::config::Settings::default();
+        show_bookmarks(&bookmarks, extended_fields, &settings);
     }
 
     #[test]
-    #[serial]
-    fn test_show_bookmarks_empty() {
+    fn given_empty_bookmark_list_when_show_then_displays_nothing() {
         println!("\n\nTEST: Empty Bookmark List\n");
         let empty_bookmarks: Vec<DisplayBookmark> = Vec::new();
-        show_bookmarks(&empty_bookmarks, DEFAULT_FIELDS);
+        let settings = crate::config::Settings::default();
+        show_bookmarks(&empty_bookmarks, DEFAULT_FIELDS, &settings);
     }
 
     #[test]
-    #[serial]
-    fn test_output_to_file() -> io::Result<()> {
+    fn given_bookmarks_when_output_to_file_then_creates_file_successfully() -> io::Result<()> {
         use std::io::Write;
 
         // Create output directory if it doesn't exist
@@ -458,7 +502,7 @@ mod display_tests {
 
         // We can't easily redirect the show_bookmarks output since it uses stderr
         // but we can create similar output manually for the table version
-        for (_, field) in DEFAULT_FIELDS.iter().enumerate() {
+        for field in DEFAULT_FIELDS.iter() {
             write!(output_file, "{} ", field)?;
         }
         writeln!(output_file)?;
