@@ -20,12 +20,10 @@ impl Embedder for DummyEmbedding {
 
 #[cfg(test)]
 mod tests {
-    use serial_test::serial;
     use super::*;
 
     #[test]
-    #[serial]
-    fn test_dummy_embedding_returns_none() {
+    fn given_text_input_when_create_dummy_embedding_then_returns_none() {
         let dummy = DummyEmbedding;
         let result = dummy.embed("test text").unwrap();
         assert!(result.is_none());
