@@ -7,12 +7,10 @@ use diesel::sql_types::{Integer, Text};
 use std::collections::HashSet;
 use tracing::{debug, error, info, instrument};
 
-use diesel::QueryableByName;
-
 use super::connection::{ConnectionPool, PooledConnection};
 use super::error::{SqliteRepositoryError, SqliteResult};
 use crate::domain::bookmark::Bookmark;
-use crate::domain::error::DomainError;
+use crate::domain::error::{DomainError, RepositoryError};
 use crate::domain::repositories::query::{
     AllTagsSpecification, AnyTagSpecification, BookmarkQuery, SortDirection,
 };
@@ -22,6 +20,7 @@ use crate::infrastructure::repositories::sqlite::model::{
     DbBookmark, DbBookmarkChanges, IdResult, NewBookmark, TagsFrequency,
 };
 use crate::infrastructure::repositories::sqlite::schema::bookmarks::dsl;
+use diesel::QueryableByName;
 
 #[derive(Debug, QueryableByName)]
 struct TableSchema {
@@ -60,7 +59,8 @@ impl SqliteBookmarkRepository {
     /// Create a new SQLite repository with the provided database URL
     #[instrument(skip_all, level = "debug")]
     pub fn from_url(database_url: &str) -> SqliteResult<Self> {
-        let pool = super::connection::init_pool(database_url)?;
+        let pool = super::connection::init_pool(database_url)
+            .map_err(|e| e.context("initializing database connection pool"))?;
         Ok(Self { pool })
     }
 
@@ -70,6 +70,7 @@ impl SqliteBookmarkRepository {
         self.pool
             .get()
             .map_err(|e| SqliteRepositoryError::ConnectionPoolError(e.to_string()))
+            .map_err(|e| e.context("getting database connection from pool"))
     }
 
     /// Cleans the table by deleting all bookmarks except ID 1
@@ -80,7 +81,8 @@ impl SqliteBookmarkRepository {
         // sql_query("DELETE FROM bookmarks WHERE id != 1;")
         sql_query("DELETE FROM bookmarks;")
             .execute(&mut conn)
-            .map_err(SqliteRepositoryError::DatabaseError)?;
+            .map_err(SqliteRepositoryError::DatabaseError)
+            .map_err(|e| e.context("executing table cleanup query"))?;
 
         debug!("Cleaned table.");
         Ok(())
@@ -89,12 +91,13 @@ impl SqliteBookmarkRepository {
     /// Convert a database model to a domain entity
     #[instrument(skip_all, level = "trace")]
     fn to_domain_model(&self, db_bookmark: DbBookmark) -> SqliteResult<Bookmark> {
-        // Convert stored timestamp to DateTime<Utc>
-        let created_at =
-            DateTime::<Utc>::from_naive_utc_and_offset(db_bookmark.last_update_ts, Utc);
         let updated_at =
             DateTime::<Utc>::from_naive_utc_and_offset(db_bookmark.last_update_ts, Utc);
 
+        let created_at = db_bookmark
+            .created_ts
+            .map(|dt| DateTime::<Utc>::from_naive_utc_and_offset(dt, Utc));
+
         // Create bookmark from storage data
         Bookmark::from_storage(
             db_bookmark.id,
@@ -108,6 +111,9 @@ impl SqliteBookmarkRepository {
             db_bookmark.embedding,
             db_bookmark.content_hash,
             db_bookmark.embeddable,
+            db_bookmark.file_path,
+            db_bookmark.file_mtime,
+            db_bookmark.file_hash,
         )
         .map_err(|e| {
             SqliteRepositoryError::ConversionError(format!(
@@ -120,34 +126,52 @@ impl SqliteBookmarkRepository {
     /// Convert a domain entity to a database model
     #[instrument(skip_all, level = "debug")]
     fn to_db_model(&self, bookmark: &Bookmark) -> DbBookmarkChanges {
-        DbBookmarkChanges {
+        // Create the changes with explicit Option types to ensure NULL values are handled correctly
+        let changes = DbBookmarkChanges {
             url: bookmark.url.to_string(),
             metadata: bookmark.title.to_string(),
             tags: bookmark.formatted_tags(),
             desc: bookmark.description.to_string(),
             flags: bookmark.access_count,
-            embedding: bookmark.embedding.clone(), // Get embedding from domain entity
-            content_hash: bookmark.content_hash.clone(), // Get content hash from domain entity
-            created_ts: Some(bookmark.created_at.naive_utc()),
+            embedding: bookmark.embedding.clone(),
+            content_hash: bookmark.content_hash.clone(),
+            created_ts: bookmark.created_at.map(|dt| dt.naive_utc()),
             embeddable: bookmark.embeddable,
-        }
+            file_path: bookmark.file_path.clone(),
+            file_mtime: bookmark.file_mtime,
+            file_hash: bookmark.file_hash.clone(),
+        };
+
+        debug!(
+            "Created DB model changes: embedding is null: {}, content_hash is null: {}",
+            changes.embedding.is_none(),
+            changes.content_hash.is_none()
+        );
+
+        changes
     }
 }
 
 impl BookmarkRepository for SqliteBookmarkRepository {
     #[instrument(skip_all, level = "debug")]
     fn get_by_id(&self, id: i32) -> Result<Option<Bookmark>, DomainError> {
-        let mut conn = self.get_connection()?;
+        let mut conn = self.get_connection()
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context("getting database connection for bookmark lookup"))?;
 
         let result = dsl::bookmarks
             .filter(dsl::id.eq(id))
             .first::<DbBookmark>(&mut conn)
             .optional()
-            .map_err(SqliteRepositoryError::DatabaseError)?;
+            .map_err(SqliteRepositoryError::DatabaseError)
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context(format!("querying bookmark by ID {}", id)))?;
 
         match result {
             Some(db_bookmark) => {
-                let bookmark = self.to_domain_model(db_bookmark)?;
+                let bookmark = self.to_domain_model(db_bookmark)
+                    .map_err(|e| DomainError::RepositoryError(e.into()))
+                    .map_err(|e| e.context(format!("converting database model to domain model for bookmark ID {}", id)))?;
                 Ok(Some(bookmark))
             }
             None => Ok(None),
@@ -156,7 +180,9 @@ impl BookmarkRepository for SqliteBookmarkRepository {
 
     #[instrument(skip_all, level = "debug")]
     fn get_by_url(&self, url: &str) -> Result<Option<Bookmark>, DomainError> {
-        let mut conn = self.get_connection()?;
+        let mut conn = self.get_connection()
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context("getting database connection for URL lookup"))?;
 
         // Escape special characters in URL for SQLite query
         let escaped_url = url.replace('\'', "''");
@@ -165,11 +191,15 @@ impl BookmarkRepository for SqliteBookmarkRepository {
             .filter(dsl::URL.eq(escaped_url))
             .first::<DbBookmark>(&mut conn)
             .optional()
-            .map_err(SqliteRepositoryError::DatabaseError)?;
+            .map_err(SqliteRepositoryError::DatabaseError)
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context(format!("querying bookmark by URL: {}", url)))?;
 
         match result {
             Some(db_bookmark) => {
-                let bookmark = self.to_domain_model(db_bookmark)?;
+                let bookmark = self.to_domain_model(db_bookmark)
+                    .map_err(|e| DomainError::RepositoryError(e.into()))
+                    .map_err(|e| e.context(format!("converting database model to domain model for URL: {}", url)))?;
                 Ok(Some(bookmark))
             }
             None => Ok(None),
@@ -178,79 +208,83 @@ impl BookmarkRepository for SqliteBookmarkRepository {
 
     #[instrument(skip_all, level = "debug")]
     fn search(&self, query: &BookmarkQuery) -> Result<Vec<Bookmark>, DomainError> {
-        let mut conn = self.get_connection()?;
+        let mut conn = self.get_connection()
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context("getting database connection for bookmark search"))?;
 
-        // If there's a specification, we need more complex handling
-        if query.specification.is_some() {
-            // Get all bookmarks and filter them in-memory using the specification
-            let all_bookmarks = self.get_all()?;
-
-            let filtered = all_bookmarks
-                .into_iter()
-                .filter(|bookmark| query.matches(bookmark))
-                .collect::<Vec<_>>();
-
-            // Apply sorting
-            let mut sorted = filtered;
-            if let Some(sort_direction) = query.sort_by_date {
-                match sort_direction {
-                    SortDirection::Ascending => {
-                        sorted.sort_by_key(|a| a.updated_at);
-                    }
-                    SortDirection::Descending => {
-                        sorted.sort_by(|a, b| b.updated_at.cmp(&a.updated_at));
-                    }
-                }
+        // First, handle text query with FTS if present
+        let bookmark_ids = if let Some(text_query) = &query.text_query {
+            if !text_query.is_empty() {
+                // Use FTS to get matching bookmark IDs
+                debug!("Using FTS search for query: {}", text_query);
+                self.get_bookmarks_fts(text_query)
+                    .map_err(|e| e.context(format!("performing FTS search for query: {}", text_query)))?
+            } else {
+                // Empty text query, get all IDs
+                debug!("Empty text query, retrieving all bookmark IDs");
+                self.get_all_bookmark_ids(&mut conn)
+                    .map_err(|e| DomainError::RepositoryError(e.into()))
+                    .map_err(|e| e.context("retrieving all bookmark IDs for empty text query"))?
             }
+        } else {
+            // No text query, get all IDs
+            debug!("No text query, retrieving all bookmark IDs");
+            self.get_all_bookmark_ids(&mut conn)
+                .map_err(|e| DomainError::RepositoryError(e.into()))
+                .map_err(|e| e.context("retrieving all bookmark IDs for no text query"))?
+        };
 
-            // Apply offset and limit
-            let mut result = sorted;
-            if let Some(offset) = query.offset {
-                if offset < result.len() {
-                    result = result.into_iter().skip(offset).collect();
-                } else {
-                    result = Vec::new();
-                }
-            }
-
-            if let Some(limit) = query.limit {
-                result = result.into_iter().take(limit).collect();
-            }
-
-            return Ok(result);
+        // If we have no IDs after FTS, return empty result quickly
+        if bookmark_ids.is_empty() {
+            debug!("No matching bookmarks found for text query");
+            return Ok(Vec::new());
         }
 
-        // If no specification, just apply sorting and pagination
-        // Build a query using the BoxedSelectStatement
-        let mut query_builder = dsl::bookmarks.into_boxed();
+        // Fetch the complete bookmark objects for the matching IDs
+        let bookmarks = self.get_bookmarks_by_ids(&bookmark_ids)
+            .map_err(|e| e.context("fetching complete bookmark objects by IDs"))?;
 
-        // Apply sorting
-        if let Some(sort_direction) = query.sort_by_date {
-            match sort_direction {
-                SortDirection::Ascending => {
-                    query_builder = query_builder.order(dsl::last_update_ts.asc());
-                }
-                SortDirection::Descending => {
-                    query_builder = query_builder.order(dsl::last_update_ts.desc());
-                }
-            }
+        // Apply all other filters from the query
+        let filtered_bookmarks = query.apply_non_text_filters(&bookmarks);
+
+        debug!(
+            "After filtering: {} bookmarks match the query",
+            filtered_bookmarks.len()
+        );
+        Ok(filtered_bookmarks)
+    }
+
+    #[instrument(skip_all, level = "trace")]
+    fn get_all_bookmark_ids(
+        &self,
+        conn: &mut PooledConnection,
+    ) -> Result<Vec<i32>, SqliteRepositoryError> {
+        let ids = dsl::bookmarks
+            .select(dsl::id)
+            .load::<i32>(conn)
+            .map_err(SqliteRepositoryError::DatabaseError)
+            .map_err(|e| e.context("loading all bookmark IDs from database"))?;
+
+        Ok(ids)
+    }
+
+    #[instrument(skip_all, level = "trace")]
+    fn get_bookmarks_by_ids(&self, ids: &[i32]) -> Result<Vec<Bookmark>, DomainError> {
+        if ids.is_empty() {
+            return Ok(Vec::new());
         }
 
-        // Apply pagination
-        if let Some(limit) = query.limit {
-            query_builder = query_builder.limit(limit as i64);
-        }
+        let mut conn = self.get_connection()
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context("getting database connection for bulk bookmark retrieval"))?;
 
-        if let Some(offset) = query.offset {
-            query_builder = query_builder.offset(offset as i64);
-        }
-
-        // Execute query
-        let db_bookmarks = query_builder
+        let db_bookmarks = dsl::bookmarks
+            .filter(dsl::id.eq_any(ids))
             .load::<DbBookmark>(&mut conn)
-            .map_err(SqliteRepositoryError::DatabaseError)?;
+            .map_err(SqliteRepositoryError::DatabaseError)
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context(format!("querying bookmarks by IDs: {:?}", ids)))?;
 
-        // Convert to domain models
         let bookmarks = db_bookmarks
             .into_iter()
             .filter_map(|db_bookmark| match self.to_domain_model(db_bookmark) {
@@ -267,11 +301,15 @@ impl BookmarkRepository for SqliteBookmarkRepository {
 
     #[instrument(skip_all, level = "debug")]
     fn get_all(&self) -> Result<Vec<Bookmark>, DomainError> {
-        let mut conn = self.get_connection()?;
+        let mut conn = self.get_connection()
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context("getting database connection for retrieving all bookmarks"))?;
 
         let db_bookmarks = dsl::bookmarks
             .load::<DbBookmark>(&mut conn)
-            .map_err(SqliteRepositoryError::DatabaseError)?;
+            .map_err(SqliteRepositoryError::DatabaseError)
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context("loading all bookmarks from database"))?;
 
         let mut bookmarks = Vec::new();
         for db_bookmark in db_bookmarks {
@@ -286,7 +324,9 @@ impl BookmarkRepository for SqliteBookmarkRepository {
 
     #[instrument(skip_all, level = "debug")]
     fn add(&self, bookmark: &mut Bookmark) -> Result<(), DomainError> {
-        let mut conn = self.get_connection()?;
+        let mut conn = self.get_connection()
+            .map_err(|e| DomainError::RepositoryError(e.into()))
+            .map_err(|e| e.context("getting database connection for adding bookmark"))?;
 
         // Begin transaction
         conn.transaction::<_, diesel::result::Error, _>(|conn| {
@@ -298,8 +338,11 @@ impl BookmarkRepository for SqliteBookmarkRepository {
                 flags: bookmark.access_count,
                 embedding: bookmark.embedding.clone(),
                 content_hash: bookmark.content_hash.clone(),
-                created_ts: Some(bookmark.created_at.naive_utc()),
+                created_ts: bookmark.created_at.map(|dt| dt.naive_utc()),
                 embeddable: bookmark.embeddable,
+                file_path: bookmark.file_path.clone(),
+                file_mtime: bookmark.file_mtime,
+                file_hash: bookmark.file_hash.clone(),
             };
             debug!("Inserting bookmark: {}", db_bookmark);
 
@@ -334,8 +377,8 @@ impl BookmarkRepository for SqliteBookmarkRepository {
             SqliteRepositoryError::OperationFailed("Bookmark has no ID".to_string())
         })?;
 
-        // Create the update data
         let changes = self.to_db_model(bookmark);
+        // debug!("Updating bookmark with ID {}: {:?}", id, changes);  // logs entire embedding
 
         // Update the bookmark
         let result = diesel::update(dsl::bookmarks.filter(dsl::id.eq(id)))
@@ -356,19 +399,10 @@ impl BookmarkRepository for SqliteBookmarkRepository {
 
         // Begin transaction
         conn.transaction::<bool, diesel::result::Error, _>(|conn| {
-            // Delete the bookmark
             let result = diesel::delete(dsl::bookmarks.filter(dsl::id.eq(id))).execute(conn)?;
-
             if result == 0 {
                 return Ok(false); // No bookmark was deleted
             }
-
-            // Update IDs of remaining bookmarks to maintain sequential IDs
-            sql_query("UPDATE bookmarks SET id = id - 1 WHERE id > ?")
-                .bind::<Integer, _>(id)
-                .execute(conn)?;
-
-            // Return success
             Ok(true)
         })
         .map_err(SqliteRepositoryError::DatabaseError)?;
@@ -548,7 +582,7 @@ impl BookmarkRepository for SqliteBookmarkRepository {
     ) -> Result<Vec<Bookmark>, DomainError> {
         let mut query = BookmarkQuery::new().with_sort_by_date(direction);
         if let Some(limit) = limit {
-            query = query.with_limit(limit);
+            query = query.with_limit(Option::from(limit));
         }
         self.search(&query)
     }
@@ -560,7 +594,10 @@ impl BookmarkRepository for SqliteBookmarkRepository {
         // );
         // self.search(&query)
         self.get_bookmarks(text).map_err(|e| {
-            DomainError::RepositoryError(format!("Failed to search bookmarks by text: {}", e))
+            DomainError::RepositoryError(RepositoryError::Other(format!(
+                "Failed to search bookmarks by text: {}",
+                e
+            )))
         })
     }
 
@@ -650,8 +687,16 @@ impl BookmarkRepository for SqliteBookmarkRepository {
         Ok(ids)
     }
 
-    fn exists_by_url(&self, url: &str) -> Result<bool, DomainError> {
-        self.get_by_url(url).map(|result| result.is_some())
+    #[instrument(skip(self))]
+    fn exists_by_url(&self, url: &str) -> Result<i32, DomainError> {
+        let bookmark = self.get_by_url(url)?;
+
+        match bookmark {
+            Some(bm) => bm
+                .id
+                .ok_or_else(|| DomainError::BookmarkNotFound("Bookmark ID is None".to_string())),
+            None => Ok(-1),
+        }
     }
 
     #[instrument(skip_all, level = "debug")]
@@ -659,8 +704,14 @@ impl BookmarkRepository for SqliteBookmarkRepository {
         let mut conn = self.get_connection()?;
 
         // Query for bookmarks that are embeddable but don't have embeddings
+        // Only check if embedding is NULL, ignore content_hash
         let db_bookmarks = dsl::bookmarks
-            .filter(dsl::embedding.is_null().and(dsl::embeddable.eq(true)))
+            .filter(
+                dsl::embeddable
+                    .eq(true)
+                    .and(dsl::embedding.is_null())
+                    .and(dsl::tags.not_like("%,_imported_,%")),
+            )
             .load::<DbBookmark>(&mut conn)
             .map_err(SqliteRepositoryError::DatabaseError)?;
 
@@ -679,10 +730,8 @@ impl BookmarkRepository for SqliteBookmarkRepository {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::app_state::AppState;
     use crate::domain::repositories::query::TextSearchSpecification;
     use crate::util::testing::{init_test_env, setup_test_db};
-    use serial_test::serial;
     use std::collections::HashSet;
 
     fn create_test_bookmark(
@@ -695,14 +744,12 @@ mod tests {
             .map(Tag::new)
             .collect::<Result<HashSet<_>, _>>()?;
 
-        let app_state = AppState::read_global();
-        let embedder = &*app_state.context.embedder;
-        Bookmark::new(url, title, "Test description", tag_set, embedder)
+        let embedder = crate::infrastructure::embeddings::DummyEmbedding;
+        Bookmark::new(url, title, "Test description", tag_set, &embedder)
     }
 
     #[test]
-    #[serial]
-    fn test_add_and_get_by_id() -> Result<(), DomainError> {
+    fn given_new_bookmark_when_add_and_get_by_id_then_retrieves_successfully() -> Result<(), DomainError> {
         let repo = setup_test_db();
 
         let mut bookmark = create_test_bookmark(
@@ -734,8 +781,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_get_by_url() -> Result<(), DomainError> {
+    fn given_bookmark_with_url_when_get_by_url_then_finds_bookmark() -> Result<(), DomainError> {
         let repo = setup_test_db();
 
         // Create and add a bookmark
@@ -760,8 +806,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_update() -> Result<(), DomainError> {
+    fn given_existing_bookmark_when_update_then_changes_persist() -> Result<(), DomainError> {
         let repo = setup_test_db();
 
         // Create and add a bookmark
@@ -798,8 +843,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_delete_and_reindex() -> Result<(), DomainError> {
+    fn given_existing_bookmark_when_delete_then_removes_and_reindexes() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -815,8 +859,10 @@ mod tests {
         // Directly query all bookmarks to see their state
         let all = repo.get_all()?;
         assert_eq!(all.len(), 2, "Should have 2 bookmarks before deletion");
-        assert_eq!(all[0].id, Some(1), "First bookmark should have ID 11");
-        assert_eq!(all[1].id, Some(2), "Second bookmark should have ID 12");
+
+        // Get the IDs of the bookmarks
+        // let id1 = bookmark1.id.unwrap();
+        let id2 = bookmark2.id.unwrap();
 
         // Delete first bookmark
         repo.delete(1)?;
@@ -825,11 +871,11 @@ mod tests {
         let updated = repo.get_all()?;
         assert_eq!(updated.len(), 1, "Should have 1 bookmark after deletion");
 
-        // The remaining bookmark should have ID 1 and be the second bookmark
+        // The remaining bookmark should still have its original ID
         assert_eq!(
             updated[0].id,
-            Some(1),
-            "Remaining bookmark should have ID 1"
+            Some(id2),
+            "Remaining bookmark should keep its original ID"
         );
         assert_eq!(
             updated[0].url, "https://second.com",
@@ -840,8 +886,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_search() -> Result<(), DomainError> {
+    fn given_search_query_when_search_then_returns_matching_bookmarks() -> Result<(), DomainError> {
         let repo = setup_test_db();
 
         // Add test bookmarks
@@ -884,8 +929,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_get_all_tags() -> Result<(), DomainError> {
+    fn given_bookmarks_with_tags_when_get_all_tags_then_returns_unique_tags() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -923,8 +967,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_get_related_tags() -> Result<(), DomainError> {
+    fn given_tag_query_when_get_related_tags_then_returns_cooccurring_tags() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -962,8 +1005,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_get_random() -> Result<(), DomainError> {
+    fn given_bookmarks_exist_when_get_random_then_returns_random_selection() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -993,8 +1035,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_get_without_embeddings() -> Result<(), DomainError> {
+    fn given_bookmarks_without_embeddings_when_get_then_returns_filtered_list() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -1018,8 +1059,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_exists_by_url() -> Result<(), DomainError> {
+    fn given_url_when_exists_by_url_then_returns_existence_status() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -1030,11 +1070,11 @@ mod tests {
 
         // Check existing URL
         let exists = repo.exists_by_url("https://exists-test.com")?;
-        assert!(exists);
+        assert_eq!(exists, 1);
 
         // Check non-existing URL
         let not_exists = repo.exists_by_url("https://does-not-exist.com")?;
-        assert!(!not_exists);
+        assert_eq!(not_exists, -1);
 
         Ok(())
     }
@@ -1078,77 +1118,7 @@ mod tests {
     // }
 
     #[test]
-    #[serial]
-    fn test_database_compaction_after_delete() -> Result<(), DomainError> {
-        let repo = setup_test_db();
-        repo.empty_bookmark_table()?;
-
-        // Create several bookmarks in sequence
-        let mut bm1 = create_test_bookmark("First", "https://first.com", vec!["test"])?;
-        let mut bm2 = create_test_bookmark("Second", "https://second.com", vec!["test"])?;
-        let mut bm3 = create_test_bookmark("Third", "https://third.com", vec!["test"])?;
-        let mut bm4 = create_test_bookmark("Fourth", "https://fourth.com", vec!["test"])?;
-
-        repo.add(&mut bm1)?;
-        repo.add(&mut bm2)?;
-        repo.add(&mut bm3)?;
-        repo.add(&mut bm4)?;
-
-        // Verify IDs are sequential
-        let all_before = repo.get_all()?;
-        let ids_before: Vec<i32> = all_before.iter().filter_map(|b| b.id).collect();
-
-        // Make sure we have at least 4 bookmarks
-        assert!(ids_before.len() >= 4, "Should have at least 4 bookmarks");
-
-        // Delete bookmark with ID 2
-        let id_to_delete = 2;
-        repo.delete(id_to_delete)?;
-
-        // Get all bookmarks again
-        let all_after = repo.get_all()?;
-
-        // Check that IDs have been compacted (no gaps)
-        let ids_after: Vec<i32> = all_after.iter().filter_map(|b| b.id).collect();
-
-        // Should have one fewer bookmark
-        assert_eq!(
-            ids_after.len(),
-            ids_before.len() - 1,
-            "Should have one fewer bookmark"
-        );
-
-        // The IDs should be sequential without gaps
-        for i in 1..=ids_after.len() {
-            assert!(
-                ids_after.contains(&(i as i32)),
-                "Missing ID {} after compaction",
-                i
-            );
-        }
-
-        // Specifically, check that the bookmark that was at ID 3 is now at ID 2
-        let former_id3 = all_before
-            .iter()
-            .find(|b| b.id == Some(3))
-            .map(|b| b.url.to_string());
-
-        let new_id2 = all_after
-            .iter()
-            .find(|b| b.id == Some(2))
-            .map(|b| b.url.to_string());
-
-        assert_eq!(
-            former_id3, new_id2,
-            "Bookmark formerly at ID 3 should now be at ID 2"
-        );
-
-        Ok(())
-    }
-
-    #[test]
-    #[serial]
-    fn test_get_by_invalid_id() -> Result<(), DomainError> {
+    fn given_invalid_id_when_get_by_id_then_returns_none() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -1162,8 +1132,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_get_all_tags_as_vector() -> Result<(), DomainError> {
+    fn given_tagged_bookmarks_when_get_all_tags_as_vector_then_returns_sorted_tags() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -1206,8 +1175,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_check_schema_migrations_exists() -> Result<(), DomainError> {
+    fn given_database_when_check_schema_migrations_then_verifies_existence() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -1231,7 +1199,7 @@ mod tests {
 
         // Initially the table might not exist in a fresh test DB
         // The important thing is that the query executes correctly
-        println!("Schema migrations table exists: {}", exists);
+        eprintln!("Schema migrations table exists: {}", exists);
 
         Ok(())
     }
@@ -1244,8 +1212,7 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_check_embedding_column_exists() -> Result<(), DomainError> {
+    fn given_database_when_check_embedding_column_then_verifies_existence() -> Result<(), DomainError> {
         let repo = setup_test_db();
         repo.empty_bookmark_table()?;
 
@@ -1273,11 +1240,336 @@ mod tests {
     }
 
     #[test]
-    #[serial]
-    fn test_setup_test_db() {
+    fn given_test_environment_when_setup_test_db_then_creates_database() {
         let _ = init_test_env();
         let repo = setup_test_db();
         assert!(repo.get_connection().is_ok());
         print_db_schema(&repo);
     }
+
+    #[test]
+    fn given_bookmarks_exist_when_get_all_ids_then_returns_id_list() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+        let mut conn = repo.get_connection()?;
+
+        // Act
+        let ids = repo.get_all_bookmark_ids(&mut conn)?;
+
+        // Assert
+        assert!(!ids.is_empty(), "Should return at least some bookmark IDs");
+
+        // Verify count matches total bookmarks
+        let all_bookmarks = repo.get_all()?;
+        assert_eq!(
+            ids.len(),
+            all_bookmarks.len(),
+            "Number of IDs should match number of bookmarks"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_valid_ids_when_get_bookmarks_by_ids_then_returns_bookmarks() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+        let mut conn = repo.get_connection()?;
+
+        // Get a subset of IDs (first 3)
+        let all_ids = repo.get_all_bookmark_ids(&mut conn)?;
+        let subset_ids: Vec<i32> = all_ids.into_iter().take(3).collect();
+
+        // Act
+        let bookmarks = repo.get_bookmarks_by_ids(&subset_ids)?;
+
+        // Assert
+        assert_eq!(
+            bookmarks.len(),
+            subset_ids.len(),
+            "Should return exactly the number of bookmarks for the provided IDs"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_empty_id_list_when_get_bookmarks_by_ids_then_returns_empty() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+        let empty_ids: Vec<i32> = Vec::new();
+
+        // Act
+        let bookmarks = repo.get_bookmarks_by_ids(&empty_ids)?;
+
+        // Assert
+        assert!(
+            bookmarks.is_empty(),
+            "Should return empty vector for empty IDs list"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_nonexistent_ids_when_get_bookmarks_by_ids_then_returns_empty() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+        let nonexistent_ids = vec![99999, 99998, 99997]; // IDs that shouldn't exist
+
+        // Act
+        let bookmarks = repo.get_bookmarks_by_ids(&nonexistent_ids)?;
+
+        // Assert
+        assert!(
+            bookmarks.is_empty(),
+            "Should return empty vector for nonexistent IDs"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_mixed_valid_invalid_ids_when_get_bookmarks_then_returns_valid_only() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+        let mut conn = repo.get_connection()?;
+
+        // Get some valid IDs
+        let valid_ids: Vec<i32> = repo
+            .get_all_bookmark_ids(&mut conn)?
+            .into_iter()
+            .take(2)
+            .collect();
+
+        // Create a list with both valid and invalid IDs
+        let mut mixed_ids = valid_ids.clone();
+        mixed_ids.push(99999); // Add a nonexistent ID
+
+        // Act
+        let bookmarks = repo.get_bookmarks_by_ids(&mixed_ids)?;
+
+        // Assert
+        assert_eq!(
+            bookmarks.len(),
+            valid_ids.len(),
+            "Should return only bookmarks for valid IDs"
+        );
+
+        // Check that each returned bookmark has one of the valid IDs
+        for bookmark in &bookmarks {
+            assert!(
+                valid_ids.contains(&bookmark.id.unwrap()),
+                "Returned bookmark should have a valid ID"
+            );
+        }
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_text_query_only_when_search_then_returns_matching_results() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+
+        // Create a query with just a text search
+        let query = BookmarkQuery::new().with_text_query(Some("Google"));
+
+        // Act
+        let results = repo.search(&query)?;
+
+        // Assert
+        assert!(
+            !results.is_empty(),
+            "Should find bookmarks matching text query"
+        );
+
+        // Every result should contain "Google" somewhere
+        // Note: This is an approximation since FTS might use stemming, etc.
+        let has_match = results
+            .iter()
+            .any(|b| b.title.contains("Google") || b.url.contains("google"));
+
+        assert!(
+            has_match,
+            "At least one result should contain the search text"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_empty_text_query_when_search_then_returns_all_results() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+
+        // Create a query with an empty text search
+        let query = BookmarkQuery::new().with_text_query(Some(""));
+
+        // Act
+        let results = repo.search(&query)?;
+
+        // Assert
+        assert!(
+            !results.is_empty(),
+            "Empty text query should return all bookmarks"
+        );
+
+        // Results should match get_all
+        let all_bookmarks = repo.get_all()?;
+        assert_eq!(
+            results.len(),
+            all_bookmarks.len(),
+            "Empty text query should return all bookmarks"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_no_text_query_when_search_then_returns_all_results() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+
+        // Create a query with no text search
+        let query = BookmarkQuery::new();
+
+        // Act
+        let results = repo.search(&query)?;
+
+        // Assert
+        assert!(
+            !results.is_empty(),
+            "No text query should return all bookmarks"
+        );
+
+        // Results should match get_all
+        let all_bookmarks = repo.get_all()?;
+        assert_eq!(
+            results.len(),
+            all_bookmarks.len(),
+            "No text query should return all bookmarks"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_text_and_tag_filters_when_search_then_returns_filtered_results() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+
+        // Create a tag that exists in sample data
+        let mut tags = HashSet::new();
+        tags.insert(Tag::new("aaa")?);
+
+        // Create a query with both text and tag filters
+        let query = BookmarkQuery::new()
+            .with_text_query(Some("TEST"))
+            .with_tags_all(Some(&tags));
+
+        // Act
+        let results = repo.search(&query)?;
+
+        // Assert
+        // Each result should have the specified tag
+        for bookmark in &results {
+            assert!(
+                bookmark.tags.contains(&Tag::new("aaa")?),
+                "Search results should respect tag filtering"
+            );
+        }
+
+        // Compare with results from a query with just the text
+        let text_only_query = BookmarkQuery::new().with_text_query(Some("TEST"));
+        let text_only_results = repo.search(&text_only_query)?;
+
+        // The filtered results should be a subset of the text-only results
+        assert!(
+            results.len() <= text_only_results.len(),
+            "Adding tag filters should return same or fewer results"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_nonmatching_text_query_when_search_then_returns_empty() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+
+        // Create a query with a text search that shouldn't match anything
+        let query =
+            BookmarkQuery::new().with_text_query(Some("ThisShouldNotMatchAnything12345XYZ"));
+
+        // Act
+        let results = repo.search(&query)?;
+
+        // Assert
+        assert!(
+            results.is_empty(),
+            "Non-matching text query should return empty results"
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn given_mixed_filters_when_search_then_applies_all_criteria() -> Result<(), DomainError> {
+        // Arrange
+        let repo = setup_test_db();
+
+        // Create a complex query with multiple filter types
+        let mut all_tags = HashSet::new();
+        all_tags.insert(Tag::new("aaa")?);
+
+        let mut any_tags = HashSet::new();
+        any_tags.insert(Tag::new("bbb")?);
+        any_tags.insert(Tag::new("xxx")?);
+
+        let query = BookmarkQuery::new()
+            .with_text_query(Some("TEST"))
+            .with_tags_all(Some(&all_tags))
+            .with_tags_any(Some(&any_tags))
+            .with_sort_by_date(SortDirection::Descending)
+            .with_limit(Some(5));
+
+        // Act
+        let results = repo.search(&query)?;
+
+        // Assert
+        // Each result should match all filter criteria
+        for bookmark in &results {
+            // Should have the "all" tag
+            assert!(
+                bookmark.tags.contains(&Tag::new("aaa")?),
+                "Results should have the 'all' tag"
+            );
+
+            // Should have at least one of the "any" tags
+            assert!(
+                bookmark.tags.contains(&Tag::new("bbb")?)
+                    || bookmark.tags.contains(&Tag::new("xxx")?),
+                "Results should have at least one of the 'any' tags"
+            );
+        }
+
+        // Should respect the limit
+        assert!(
+            results.len() <= 5,
+            "Results should respect the limit parameter"
+        );
+
+        // If there are multiple results, they should be in descending order
+        if results.len() > 1 {
+            for i in 0..results.len() - 1 {
+                assert!(
+                    results[i].updated_at >= results[i + 1].updated_at,
+                    "Results should be sorted in descending order"
+                );
+            }
+        }
+
+        Ok(())
+    }
 }
