@@ -13,7 +13,11 @@ pub mod infrastructure;
 pub mod app_state;
 pub mod cli;
 pub mod config;
+pub mod exitcode;
 pub mod util;
 
+// LSP module (optional)
+pub mod lsp;
+
 #[cfg(test)]
 mod tests {}
